﻿package {

	import flash.display.*;
	import flash.events.*;
	import flash.text.*;
	import flash.utils.describeType;

	public class QuickUI extends Sprite {

		private var _rows:Number;
		private var _currentObject:Object;
		private var _txtWidth:Number=100;
		private var _spacing:Number=10;

        private var _textFormat:TextFormat;

		public function QuickUI() {
			_rows=3;
			_textFormat = new TextFormat("_sans", 11);
		}
		
		public static function getValue(v:*):*{
			if (v == "false"){
				v = 0
			}else if (v == "true"){
				v = 1;
			}
			return v;
		}
		
		public function set rows(val:int):void {
			_rows=val;
		}
		
		public function get currentObject():Object {
			return _currentObject;
		}
		
		public function updateObject(obj:Object):void{
			if (obj is TextField){
				_currentObject[obj.name] = obj.text;
			}else if (obj is QuickCheckBox){
				_currentObject[obj.name] = obj.checked;
			}
		}
		
		var _windowDefaults:Object = {title:"Window", x:0, y:0};
		public function window(params:Object=null):void{
			var p:Object = setDefaults(_windowDefaults, params);
			graphics.clear();
			if (!p.width){
				p.width = width;
			}
			if (!p.height){
				p.height = height;
			}
			
			graphics.lineStyle(0,0x666666);
			graphics.beginFill(0xCCCCCC);
			graphics.drawRect(0,0,p.width + 20, 20);
			graphics.drawRect(0,20,p.width + 20, p.height +30);
			var txt:TextField = new TextField();
			txt.defaultTextFormat = _textFormat;
			txt.text = p.title;
			txt.multiline = false;
			txt.selectable = false;
			txt.autoSize = TextFieldAutoSize.LEFT;
			txt.x = txt.y = 2;
			addChild(txt);
		}
		private function setDefaults(defaults:Object, params:Object=null):Object {
			if (params==null) {
				params = new Object();
			}
			for (var key:String in defaults) {
				if (params[key]==null) {
					params[key]=defaults[key];
				}
			}
			return params;
		}

         
		private var _inc:int = 0;
		
		private var _include:Vector.<String> = new Vector.<String>();
		private var _labels:Vector.<String> = new Vector.<String>();
		public function add(name:String, label:String=null):void{
			_include.push(name);
			if (label == null){
				label = name;
			}
			_labels.push(label);
		}
		private var _exclude:Vector.<String> = new Vector.<String>();
		public function exclude(name:String):void{
			_exclude.push(name);
		}
		public function create(obj:Object, labels:Object=null):void {
			if (labels == null){
				labels = {};
			}
			while(this.numChildren > 0){
				removeChildAt(0);
			}
			_currentObject=obj;
			var desc:XML = describeType(obj);
			var i:int = 0;
			var j:int = 0;
			var v:XML;
			 
			var newDesc:XML;
			if (_include.length > 0){
				newDesc  = new XML(<type></type>);
				for (i = 0; i<_include.length; i++){
					for (j = 0; j<desc.variable.length(); j++){
						 
						 if (desc.variable[j].@name == _include[i]){
							 
							 newDesc.appendChild(desc.variable[j]);
						 }
					 }
					 for (j = 0; j<desc.accessor.length(); j++){
						 
						 if (desc.accessor[j].@name == _include[i]){
							 desc.accessor[j].@label = _labels[i];
							 
							 newDesc.appendChild(desc.accessor[j]);
							 
						 }
					 }
				}
				 
				desc = newDesc;
			}
			// remove any excluded properties from desc XML
			if (_exclude.length > 0){
				 
				for (i = 0; i<_exclude.length; i++){
					for (j = 0; j<desc.variable.length(); j++){
						 
						 if (desc.variable[j].@name == _exclude[i]){
							 
							delete desc.variable[j];
						 }
					 }
					 for (j = 0; j<desc.accessor.length(); j++){
						 
						 if (desc.accessor[j].@name == _exclude[i]){
							 
							delete desc.accessor[j];
						 }
					 }
				}
			}
			_inc = 0;
			// public vars
			for each (v in desc.variable) {
				 evaluateProperty(obj, labels, v);
			}
			// getters
			for each (v in desc.accessor) {
				evaluateProperty(obj, labels, v);
			}
			// erase exclude vector
			_exclude = new Vector.<String>();
			_include = new Vector.<String>();
		}
		
		private function evaluateProperty(obj:Object, labels:Object, v:XML):void{
			var nodeType:String = v.localName();
			var value:* 
			var label:String;
			if (nodeType == "variable"){
				value = obj[v.@name];
				
				if (value == null){
				  return;
			     }
				 label = labels[v.@name];
					 if (label){
					   v.@label = labels[v.@name];
					 }
				createUI(obj, v, value);
			}else if (nodeType == "accessor"){
				if (v.@access=="readwrite"){
					value = obj[v.@name];
					if (value == null){
					   return;
					 }
					 label = labels[v.@name];
					 if (label){
					   v.@label = labels[v.@name];
					 }
					createUI(obj, v, value);
				}
			}
		}
		private function createUI(obj:Object, v:XML, value:*):void{
			if (v.@type == "Boolean"){
				 
				 makeCheck(v.@label, v.@name, value);
			}else if (v.@type == "Number"){
				 makeInput(v.@label, v.@name, value);
				
			}else if (v.@type == "String"){
				 makeInput(v.@label, v.@name, value);
				
				 
			}else{ 
			    // Object - more to come here, possibly launch another QuickUI instance
			    var objType:String = String(v.@type).split("::")[1];
				//trace(objType);
				//trace(v.toXMLString());
			}
		}
		private function makeCheck(txt:String, name:String, value:*):void {
			var label:TextField = new TextField();
			if (txt=="") {
				txt=name;
			}
			var cb:QuickCheckBox = new QuickCheckBox();
			cb.name = name;
			cb.value = value;
			cb.x = 15 + int(_inc / _rows) * (_txtWidth + _spacing);
			cb.y = 55 + (_inc % _rows) * (50 + _spacing);
			label.defaultTextFormat = _textFormat;
			label.text=txt;
			label.height=20;
			label.x = cb.x + 15
			label.y = cb.y - 5;
			addChild(label);
			
			addChild(cb);
			_inc++;
		}
		
		private function makeInput(txt:String, name:String, value:*):void {
			var label:TextField = new TextField();
			if (txt=="") {
				txt=name;
			}
			label.defaultTextFormat = _textFormat;
			label.text=txt;
			label.height=20;
			label.x = 10 + int(_inc / _rows) * (_txtWidth + _spacing);
			label.y = 30 + (_inc % _rows) * (50 + _spacing);
			addChild(label);
			var input:TextField = new TextField();
			input.name=name;
			input.background = true;
			input.type=TextFieldType.INPUT;
			input.width=_txtWidth;
			input.height=20;
			input.text=value;

			input.border=true;
			input.x=label.x;
			input.y=label.y+label.height;
			addChild(input);
			_inc++;
		}
	}
}